import SchemaItem from '../SchemaItem'
import { SchemaInfer, ValidationResult } from '../types'

export default class SchemaNullable<Type extends SchemaItem> extends SchemaItem<SchemaInfer<Type> | undefined> {
	public constructor(public readonly child: Type) { super([child]) }

	public unwrap(): Type {
		return this.child
	}

	public override parse(input: unknown, options?: { fast?: boolean }): ValidationResult<SchemaInfer<Type> | undefined> {
		if (this.isNull(input)) {
			return {
				valid: true,
				object: undefined
			}
		}

		return this.child.parse(input, options)
	}

	public override isOfType(input: unknown): input is SchemaInfer<Type> | undefined {
		return Array.isArray(input)
	}

	private isNull(value: unknown): value is undefined | null {
		return typeof value === 'undefined' || value === null
	}
}
